@@ -2,7 +2,7 @@
 #ifndef _GLIBMM_MAIN_H
 #define _GLIBMM_MAIN_H
 
-/* $Id: main.h 213 2005-03-14 15:22:13Z murrayc $ */
+/* $Id: main.h 589 2008-02-07 22:49:51Z murrayc $ */
 
 /* Copyright (C) 2002 The gtkmm Development Team
  *
@@ -30,6 +30,7 @@
 #include <glibmmconfig.h>
 #include <glibmm/refptr.h>
 #include <glibmm/timeval.h>
+#include <glibmm/priorities.h>
 
 GLIBMM_USING_STD(vector)
 
@@ -47,41 +48,6 @@ class IOChannel;
  * @{
  */
 
-enum
-{
-  /*! Use this for high priority event sources.  It is not used within
-   * GLib or GTK+.<br><br>
-   */
-  PRIORITY_HIGH = -100,
-
-  /*! Use this for default priority event sources.  In glibmm this
-   * priority is used by default when installing timeout handlers with
-   * SignalTimeout::connect().  In GDK this priority is used for events
-   * from the X server.<br><br>
-   */
-  PRIORITY_DEFAULT = 0,
-
-  /*! Use this for high priority idle functions.  GTK+ uses
-   * <tt>PRIORITY_HIGH_IDLE&nbsp;+&nbsp;10</tt> for resizing operations, and
-   * <tt>PRIORITY_HIGH_IDLE&nbsp;+&nbsp;20</tt> for redrawing operations.
-   * (This is done to ensure that any pending resizes are processed before
-   * any pending redraws, so that widgets are not redrawn twice unnecessarily.)
-   * <br><br>
-   */
-  PRIORITY_HIGH_IDLE = 100,
-
-  /*! Use this for default priority idle functions.  In glibmm this priority is
-   * used by default when installing idle handlers with SignalIdle::connect().
-   * <br><br>
-   */
-  PRIORITY_DEFAULT_IDLE = 200,
-
-  /*! Use this for very low priority background tasks.  It is not used within
-   * GLib or GTK+.
-   */
-  PRIORITY_LOW = 300
-};
-
 
 /** A bitwise combination representing an I/O condition to watch for on an
  * event source.
@@ -166,6 +132,18 @@ public:
 #endif
 
   /** Connects a timeout handler.
+   *
+   * Note that timeout functions may be delayed, due to the processing of other
+   * event sources. Thus they should not be relied on for precise timing.
+   * After each call to the timeout function, the time of the next
+   * timeout is recalculated based on the current time and the given interval
+   * (it does not try to 'catch up' time lost in delays).
+   *
+   * If you want to have a timer in the "seconds" range and do not care
+   * about the exact time of the first call of the timer, use the
+   * connect_seconds() function; this function allows for more
+   * optimizations and more efficient system power usage.
+   *
    * @code
    * Glib::signal_timeout().connect(sigc::ptr_fun(&timeout_handler), 1000);
    * @endcode
@@ -175,13 +153,45 @@ public:
    * timeout_source->connect(sigc::ptr_fun(&timeout_handler));
    * timeout_source->attach(Glib::MainContext::get_default());
    * @endcode
-   * @param slot A slot to call when @a interval elapsed.
+   * @param slot A slot to call when @a interval has elapsed.
    * @param interval The timeout in milliseconds.
    * @param priority The priority of the new event source.
    * @return A connection handle, which can be used to disconnect the handler.
    */
   sigc::connection connect(const sigc::slot<bool>& slot, unsigned int interval,
                            int priority = PRIORITY_DEFAULT);
+
+
+  /** Connects a timeout handler with whole second granularity.
+   *
+   * Unlike connect(), this operates at whole second granularity.
+   * The initial starting point of the timer is determined by the implementation
+   * and the implementation is expected to group multiple timers together so that
+   * they fire all at the same time.
+   *
+   * To allow this grouping, the @a interval to the first timer is rounded
+   * and can deviate up to one second from the specified interval.
+   * Subsequent timer iterations will generally run at the specified interval.
+   *
+   * @code
+   * Glib::signal_timeout().connect(sigc::ptr_fun(&timeout_handler), 1000);
+   * @endcode
+   * is equivalent to:
+   * @code
+   * const Glib::RefPtr<Glib::TimeoutSource> timeout_source = Glib::TimeoutSource::create(1000);
+   * timeout_source->connectseconds(sigc::ptr_fun(&timeout_handler));
+   * timeout_source->attach(Glib::MainContext::get_default());
+   * @endcode
+   * @param slot A slot to call when @a interval has elapsed.
+   * @param interval The timeout in seconds.
+   * @param priority The priority of the new event source.
+   * @return A connection handle, which can be used to disconnect the handler.
+   *
+   * @newin2p14
+   */
+  sigc::connection connect_seconds(const sigc::slot<bool>& slot, unsigned int interval,
+                           int priority = PRIORITY_DEFAULT);
+
 private:
   GMainContext* context_;
 
