@@ -291,14 +291,17 @@ sub append_parameter_docs($$)
 
   foreach my $param (@param_names)
   {
-    my $desc = $$param_descriptions->{$param};
-    
-    $param =~ s/([a-zA-Z0-9]*(_[a-zA-Z0-9]+)*)_?/$1/g;
-    DocsParser::convert_docs_to_cpp($obj_function, \$desc);
-    if(length($desc) > 0)
+    if ($param ne "error" ) #We wrap GErrors as exceptions, so ignore these.
     {
-      $desc  .= '.' unless($desc =~ /(?:^|\.)$/);
-      $$text .= "\n\@param ${param} \u${desc}";
+      my $desc = $$param_descriptions->{$param};
+    
+      $param =~ s/([a-zA-Z0-9]*(_[a-zA-Z0-9]+)*)_?/$1/g;
+      DocsParser::convert_docs_to_cpp($obj_function, \$desc);
+      if(length($desc) > 0)
+      {
+        $desc  .= '.' unless($desc =~ /(?:^|\.)$/);
+        $$text .= "\n\@param ${param} \u${desc}";
+      }
     }
   }
 }
@@ -350,7 +353,7 @@ sub convert_tags_to_doxygen($)
 
     # Some argument names are suffixed by "_" -- strip this.
     # gtk-doc uses @thearg, but doxygen uses @a thearg.
-    s" ?\@([a-zA-Z0-9]*(_[a-zA-Z0-9]+)*)_?\b" \@a $1 "g;
+    s" ?\@([a-zA-Z0-9]*(_[a-zA-Z0-9]+)*)_?\b" \@a $1"g;
     s"^Note ?\d?: "\@note "mg;
 
     s"&lt;/?programlisting&gt;""g;
@@ -371,6 +374,10 @@ sub convert_tags_to_doxygen($)
     s"Since: 2\.6"\@newin2p6"mg;
     s"Since: 2\.8"\@newin2p8"mg;
     s"Since: 2\.10"\@newin2p10"mg;
+    s"Since: 2\.12"\@newin2p12"mg;
+    s"Since: 2\.14"\@newin2p14"mg;
+    s"Since: 2\.16"\@newin2p16"mg;
+    s"Since: 2\.18"\@newin2p18"mg;
 
     s"\b-&gt;\b"->"g;
 
@@ -406,6 +413,7 @@ sub substitute_identifiers($$)
     # Undo wrong substitutions.
     s/\bHas::/HAS_/g;
     s/\bNo::/NO_/g;
+    s/\bG:://g; #Rename G::Something to Something. Doesn't seem to work. murrayc.
 
     # Replace C function names with C++ counterparts.
     s/\b([a-z]+_[a-z][a-z\d_]+) ?\(\)/&DocsParser::substitute_function($doc_func, $1)/eg;
