@@ -98,6 +98,7 @@ sub parse_and_build_output($)
     if ($token eq '"')         { $objOutputter->append($self->on_string_literal()); next; }
     if ($token eq '//')        { $objOutputter->append($self->on_comment_cpp()); next; }
     if ($token eq '/*')        { $objOutputter->append($self->on_comment_c()); next; }
+    if ($token eq '/**')       { $self->on_comment_doxygen(); next; }
 
     # handle #m4begin ... #m4end
     if ($token eq "#m4begin")  { $objOutputter->append($self->on_m4_section()); next;}
@@ -111,7 +112,7 @@ sub parse_and_build_output($)
     if ($token eq "_WRAP_METHOD")     { $self->on_wrap_method(); next;}
     if ($token eq "_WRAP_METHOD_DOCS_ONLY")     { $self->on_wrap_method_docs_only(); next;}
     if ($token eq "_WRAP_CORBA_METHOD")     { $self->on_wrap_corba_method(); next;} #Used in libbonobo*mm.
-    if ($token eq "_WRAP_SIGNAL") { $self->on_wrap_signal(); next;}
+    if ($token eq "_WRAP_SIGNAL") { $self->on_wrap_signal(0); next;}
     if ($token eq "_WRAP_PROPERTY") { $self->on_wrap_property(); next;}
     if ($token eq "_WRAP_VFUNC") { $self->on_wrap_vfunc(); next;}
     if ($token eq "_WRAP_CTOR")   { $self->on_wrap_ctor(); next;}
@@ -119,6 +120,7 @@ sub parse_and_build_output($)
 
     if ($token eq "_WRAP_ENUM")   { $self->on_wrap_enum(); next;}
     if ($token eq "_WRAP_GERROR") { $self->on_wrap_gerror(); next;}
+    if ($token eq "_IMPLEMENTS_INTERFACE") { $self->on_implements_interface(); next;}
 
     my $prefix_class = "_CLASS_"; # e.g. _CLASS_GTKOBJECT
     my $token_prefix = substr($token, 0, length($prefix_class));
@@ -193,6 +195,43 @@ sub extract_token($)
   return "";
 }
 
+### Returns the next token, but does not remove it from the queue, so that
+# extract_token will return it again.
+# $string peek_token()
+sub peek_token($)
+{
+  my ($self) = @_;
+
+  while ( scalar(@tokens) )
+  {
+    $_ = $tokens[0];
+
+    # skip empty tokens
+    if(!defined($_) or $_ eq "")
+    {
+      shift @tokens;
+    }
+    # eat line statements. TODO: e.g.?
+    elsif ( /^#l (\S+)\n/)
+    {
+      $$self{line_num} = $1;
+      shift @tokens;
+    }
+    # eat file statements. TODO: e.g.?
+    elsif ( /^#f (\S+)\n/)
+    {
+      $$self{filename} = $1;
+      shift @tokens;
+    }
+    else
+    {
+      return $_;
+    }
+  }
+
+  return "";
+}
+
 # bool tokens_remaining()
 sub tokens_remaining($)
 {
@@ -277,6 +316,63 @@ sub on_comment_c($)
   }
 }
 
+sub on_comment_doxygen($)
+{
+  my ($self) = @_;
+
+  my $objOutputter = $$self{objOutputter};
+
+  my @out;
+  push (@out,"/**\`");
+  while ( scalar(@tokens) )
+  {
+    $_ = $self->extract_token();
+    if ($_ eq "`") { push(@out,"\'__BT__\`"); next; }
+    if ($_ eq "'") { push(@out,"\'__FT__\`"); next; }
+
+    if ($_ eq "*/")
+    {
+      push (@out,"\'*");
+      $objOutputter->append(join("", @out));
+
+      # Find next non-whitespace token, but remember whitespace so that we
+      # can print it if the next real token is not _WRAP_SIGNAL
+      my @whitespace;
+      my $next_token = $self->peek_token();
+      while ($next_token =~ /^\s*$/)
+      {
+        push(@whitespace, $self->extract_token());
+	$next_token = $self->peek_token();
+      }
+
+      # If the next token is a signal, do not close this comment, to merge
+      # this doxygen comment with the one from the signal.
+      if($next_token eq '_WRAP_SIGNAL')
+      {
+	# Extract token and process
+	$self->extract_token();
+	# Tell wrap_signal to merge automatically generated comment with
+	# already existing comment. This is why we do not close the comment
+	# here.
+        $self->on_wrap_signal(1);
+      }
+      else
+      {
+        # Something else then signal follows, so close comment normally
+        $objOutputter->append("/");
+	# And append whitespace we ignored so far
+        $objOutputter->append(join("", @whitespace));
+	# Do not extract the token so that parse_and_build_output() will
+	# process it.
+      }
+
+      last;
+    }
+
+    push (@out,$_);
+  }
+}
+
 
 ########################################
 ###  handle #m4begin ... #m4end
@@ -650,13 +746,14 @@ sub read_file($$$)
 
   # Break the file into tokens.  Token is
   #      any group of #, A to z, 0 to 9, _
+  #      /**
   #      /*
   #      *.
   #      //
   #      any char proceeded by \
   #      symbols ;{}"`'()
   #      newline
-  @tokens = split(/(\#[lf] \S+\n)|([#A-Za-z0-9_]+)|(\/\*)|(\*\/)|(\/\/)|(\\.)|([;{}"'`()])|(\n)/,
+  @tokens = split(/(\#[lf] \S+\n)|([#A-Za-z0-9_]+)|(\/\*\*)|(\/\*)|(\*\/)|(\/\/)|(\\.)|([;{}"'`()])|(\n)/,
                          $strIn);
 }
 
@@ -759,6 +856,7 @@ sub on_wrap_method($)
   # Extra stuff needed?
   $$objCfunc{deprecated} = "";
   my $deprecation_docs = "";
+  my $ifdef;
   while(scalar(@args) > 2) # If the optional ref/err/deprecated arguments are there.
   {
     my $argRef = string_trim(pop @args);
@@ -784,12 +882,16 @@ sub on_wrap_method($)
         $deprecation_docs = string_unquote(string_trim($1));
       }
     }
+    elsif($argRef =~ /^ifdef(.*)/) #If ifdef is at the start.
+    {
+    	$ifdef = $1;
+    }
   }
 
   my $commentblock = "";
   $commentblock = DocsParser::lookup_documentation($argCFunctionName, $deprecation_docs);
 
-  $objOutputter->output_wrap_meth($filename, $line_num, $objCppfunc, $objCfunc, $argCppMethodDecl, $commentblock);
+  $objOutputter->output_wrap_meth($filename, $line_num, $objCppfunc, $objCfunc, $argCppMethodDecl, $commentblock, $ifdef);
 }
 
 # void on_wrap_method_docs_only()
@@ -914,6 +1016,38 @@ sub on_wrap_ctor($)
   $objOutputter->output_wrap_ctor($filename, $line_num, $objCppfunc, $objCfunc, $argCppMethodDecl);
 }
 
+sub on_implements_interface($$)
+{
+  my ($self) = @_;
+  
+  if( !($self->check_for_eof()) )
+  {
+   return;
+  }
+
+  my $filename = $$self{filename};
+  my $line_num = $$self{line_num};
+
+  my $str = $self->extract_bracketed_text();
+  my @args = string_split_commas($str);
+
+  # handle first argument
+  my $interface = $args[0];
+
+  # Extra stuff needed?
+  my $ifdef; 
+  while(scalar(@args) > 1) # If the optional ref/err/deprecated arguments are there.
+  {
+  	my $argRef = string_trim(pop @args);
+    if($argRef =~ /^ifdef(.*)/) #If ifdef is at the start.
+    {
+    	$ifdef = $1;
+    }
+  }
+  my $objOutputter = $$self{objOutputter};
+  $objOutputter->output_implements_interface($interface, $ifdef);	
+} 
+
 sub on_wrap_create($)
 {
   my ($self) = @_;
@@ -929,9 +1063,9 @@ sub on_wrap_create($)
   $objOutputter->output_wrap_create($str, $self);
 }
 
-sub on_wrap_signal($)
+sub on_wrap_signal($$)
 {
-  my ($self) = @_;
+  my ($self, $merge_doxycomment_with_previous) = @_;
 
   if( !($self->check_for_eof()) )
   {
@@ -951,6 +1085,7 @@ sub on_wrap_signal($)
   my $bNoDefaultHandler = 0;
   my $bCustomCCallback = 0;
   my $bRefreturn = 0;
+  my $ifdef;
   
   while(scalar(@args) > 2) # If optional arguments are there.
   {
@@ -973,11 +1108,16 @@ sub on_wrap_signal($)
     if($argRef eq "refreturn")
     {
       $bRefreturn = 1;
+    }
+    
+  	elsif($argRef =~ /^ifdef(.*)/) #If ifdef is at the start.
+    {
+    	$ifdef = $1;
     }
   }
 
 
-  $self->output_wrap_signal( $argCppDecl, $argCName, $$self{filename}, $$self{line_num}, $bCustomDefaultHandler, $bNoDefaultHandler, $bCustomCCallback, $bRefreturn);
+  $self->output_wrap_signal( $argCppDecl, $argCName, $$self{filename}, $$self{line_num}, $bCustomDefaultHandler, $bNoDefaultHandler, $bCustomCCallback, $bRefreturn, $ifdef, $merge_doxycomment_with_previous);
 }
 
 # void on_wrap_vfunc()
@@ -1001,6 +1141,7 @@ sub on_wrap_vfunc($)
 
   my $refreturn = 0;
   my $refreturn_ctype = 0;
+  my $ifdef = "";
 
   # Extra ref needed?
   while(scalar(@args) > 2) # If the optional ref/err arguments are there.
@@ -1011,10 +1152,14 @@ sub on_wrap_vfunc($)
       { $refreturn = 1; }
     elsif($argRef eq "refreturn_ctype")
       { $refreturn_ctype = 1; }
+  elsif($argRef =~ /^ifdef(.*)/) #If ifdef is at the start.
+    {
+    	$ifdef = $1;
+    }
   }
 
   $self->output_wrap_vfunc($argCppDecl, $argCName, $refreturn, $refreturn_ctype,
-                           $$self{filename}, $$self{line_num});
+                           $$self{filename}, $$self{line_num}, $ifdef);
 }
 
 sub on_wrap_enum($)
@@ -1141,9 +1286,9 @@ sub output_wrap_check($$$$$$)
 
 # void output_wrap($CppDecl, $signal_name, $filename, $line_num, $bCustomDefaultHandler, $bNoDefaultHandler, $bCustomCCallback, $bRefreturn)
 # Also used for vfunc.
-sub output_wrap_signal($$$$$$$$)
+sub output_wrap_signal($$$$$$$$$)
 {
-  my ($self, $CppDecl, $signal_name, $filename, $line_num, $bCustomDefaultHandler, $bNoDefaultHandler, $bCustomCCallback, $bRefreturn) = @_;
+  my ($self, $CppDecl, $signal_name, $filename, $line_num, $bCustomDefaultHandler, $bNoDefaultHandler, $bCustomCCallback, $bRefreturn, $ifdef, $merge_doxycomment_with_previous) = @_;
   
   #Some checks:
   $self->output_wrap_check($CppDecl, $signal_name, $filename, $line_num, "WRAP_SIGNAL");
@@ -1175,23 +1320,23 @@ sub output_wrap_signal($$$$$$$$)
     }
   }
 
-  $objOutputter->output_wrap_sig_decl($filename, $line_num, $objCSignal, $objCppSignal, $signal_name, $bCustomCCallback);
+  $objOutputter->output_wrap_sig_decl($filename, $line_num, $objCSignal, $objCppSignal, $signal_name, $bCustomCCallback, $ifdef, $merge_doxycomment_with_previous);
 
   if($bNoDefaultHandler eq 0)
   {
-    $objOutputter->output_wrap_default_signal_handler_h($filename, $line_num, $objCppSignal, $objCSignal);
+    $objOutputter->output_wrap_default_signal_handler_h($filename, $line_num, $objCppSignal, $objCSignal, $ifdef);
 
     my $bImplement = 1;
     if($bCustomDefaultHandler) { $bImplement = 0; }
-    $objOutputter->output_wrap_default_signal_handler_cc($filename, $line_num, $objCppSignal, $objCSignal, $bImplement, $bCustomCCallback, $bRefreturn);
+    $objOutputter->output_wrap_default_signal_handler_cc($filename, $line_num, $objCppSignal, $objCSignal, $bImplement, $bCustomCCallback, $bRefreturn, $ifdef);
   }
 }
 
 # void output_wrap($CppDecl, $signal_name, $filename, $line_num)
 # Also used for vfunc.
-sub output_wrap_vfunc($$$$$$$)
+sub output_wrap_vfunc($$$$$$$$)
 {
-  my ($self, $CppDecl, $vfunc_name, $refreturn, $refreturn_ctype, $filename, $line_num) = @_;
+  my ($self, $CppDecl, $vfunc_name, $refreturn, $refreturn_ctype, $filename, $line_num, $ifdef) = @_;
 
   #Some checks:
   $self->output_wrap_check($CppDecl, $vfunc_name, $filename, $line_num, "VFUNC");
@@ -1226,8 +1371,8 @@ sub output_wrap_vfunc($$$$$$$)
 
   $$objCVfunc{rettype_needs_ref} = $refreturn_ctype;
 
-  $objOutputter->output_wrap_vfunc_h($filename, $line_num, $objCppVfunc, $objCVfunc);
-  $objOutputter->output_wrap_vfunc_cc($filename, $line_num, $objCppVfunc, $objCVfunc);
+  $objOutputter->output_wrap_vfunc_h($filename, $line_num, $objCppVfunc, $objCVfunc,$ifdef);
+  $objOutputter->output_wrap_vfunc_cc($filename, $line_num, $objCppVfunc, $objCVfunc, $ifdef);
 }
 
 # give some sort of weights to sorting attibutes
